import datetime
from typing import Any, List, Optional

from popoll_backend.model import Payload


class Date(Payload):
    
    def __init__(self, data: List[Any]):
        self.id: int = data[0]
        self.title: str = data[1]
        self.date: datetime.date = datetime.date.fromisoformat(data[2])
        self.time: Optional[datetime.time] = datetime.time.fromisoformat(data[3]) if data[3] else None
        
    @classmethod
    def create_table(cls):
        return """ CREATE TABLE IF NOT EXISTS dates (
            id integer PRIMARY KEY AUTOINCREMENT,
            title text NOT NULL,
            date text NOT NULL,
            time text
        ); """


#! /usr/bin/python3
import flask
import json
import logging
import logging.handlers
import os

from flask_cors import CORS
from flask_restful import Api, Resource
from functools import wraps
from typing import Any, Dict

from popoll_backend.model.payload.history import History
from popoll_backend.query.create_answer import CreateAnswer
from popoll_backend.query.create_date import CreateDate
from popoll_backend.query.create_instrument import CreateInstrument
from popoll_backend.query.create_poll import CreatePoll
from popoll_backend.query.create_user import CreateUser
from popoll_backend.query.delete_answer import DeleteAnswer
from popoll_backend.query.delete_date import DeleteDate
from popoll_backend.query.delete_user import DeleteUser
from popoll_backend.query.get_date import GetDate
from popoll_backend.query.get_dates import GetDates
from popoll_backend.query.get_instruments import GetInstruments
from popoll_backend.query.get_poll import GetPoll
from popoll_backend.query.get_search_answer import GetSearchAnswer
from popoll_backend.query.get_session import GetSession
from popoll_backend.query.get_user import GetUser
from popoll_backend.query.get_users import GetUsers
from popoll_backend.query.update_answer import UpdateAnswer
from popoll_backend.query.update_date import UpdateDate
from popoll_backend.query.update_poll import UpdatePoll
from popoll_backend.query.create_session import CreateSession
from popoll_backend.query.update_user import UpdateUser


app = flask.Flask(__name__)
CORS(app)
api = Api(app)


def body(request: flask.request, param: str, default: Any=None, mandatory=True):
    _body = json.loads(request.data)
    return _body[param] if mandatory else _body.get(param, default)



def history(f):
    @wraps(f)
    def decorated(*args, **kwargs):
        res = f(*args, **kwargs)
        os.makedirs(os.path.join('.history', kwargs["poll"]), exist_ok=True)
        logger = logging.getLogger('my_logger')
        handler = logging.handlers.RotatingFileHandler(os.path.join('.history', kwargs["poll"], f'{kwargs["poll"]}.history.log'), maxBytes=1024*1024, backupCount=10)
        logger.addHandler(handler)
        logger.warning(json.dumps(History(flask.request, res, kwargs).toJSON()))
        return res
    return decorated


class PollEndpoint(Resource):
    def get(self, poll: str): return GetPoll(poll).run()
    def post(self, poll:str): return CreatePoll(poll, body(flask.request, 'name', mandatory=False, default=poll), body(flask.request, 'instruments', mandatory=False, default=[]), body(flask.request, 'color', mandatory=False, default="#000000")).run()
    def put(self, poll:str): return UpdatePoll(poll, body(flask.request, 'name'), body(flask.request, 'color')).run()
    
    
    

class InstrumentsEndpoint(Resource):
    def get(self, poll: str) -> Dict[str, Any]: return GetInstruments(poll).run()
    def post(self, poll: str) -> int: return CreateInstrument(poll, body(flask.request, 'name')).run()







class UsersEndpoint(Resource):
    def get(self, poll: str) -> Dict[str, Any]: return GetUsers(poll, details=True).run()
    def post(self, poll: str) -> int: return CreateUser(poll, body(flask.request, 'user')['name'], body(flask.request, 'main_instrument')['id'], [i['id'] for i in body(flask.request, 'instruments')]).run()



class UserEndpoint(Resource):
    def get(self, poll: str, id:int) -> Dict[str, Any]: return GetUser(poll, id, details=True).run()
    
    @history
    def put(self, poll: str, id: int) -> int: return UpdateUser(poll, id, body(flask.request, 'user')['name'], body(flask.request, 'main_instrument')['id'], [i['id'] for i in body(flask.request, 'instruments')]).run()
    
    @history
    def delete(self, poll: str, id: int) -> int: return DeleteUser(poll, id).run()







class DatesEndpoint(Resource):
    def get(self, poll: str): return GetDates(poll).run()
    def post(self, poll: str) -> int: return CreateDate(poll, body(flask.request, 'title'), body(flask.request, 'date'), body(flask.request, 'time', mandatory=False)).run()



class DateEndpoint(Resource):
    def get(self, poll: str, id:int) -> Dict[str, Any]: return GetDate(poll, id, details=True).run()
    def put(self, poll: str, id: int) -> int: return UpdateDate(poll, id, body(flask.request, 'title'), body(flask.request, 'date'), body(flask.request, 'time', mandatory=False)).run()
    def delete(self, poll: str, id: int) -> int: return DeleteDate(poll, id).run()







class AnswersEndpoint(Resource):
    def post(self, poll: str) -> int: return CreateAnswer(poll, body(flask.request, 'user_id'), body(flask.request, 'date_id')).run()


class AnswerEndpoint(Resource):
    def put(self, poll: str, id: int) -> int: return UpdateAnswer(poll, id, body(flask.request, 'response')).run()
    def delete(self, poll: str, id: int) -> int: return DeleteAnswer(poll, id).run()

class GetAnswerEndpoint(Resource):
    def get(self, poll: str, userId: int, dateId: int): return GetSearchAnswer(poll, userId, dateId).run()


class SessionEndpoint(Resource):
    def get(self, poll: str, id: str): return GetSession(poll, id).run()
    def post(self, poll: str, id: str): return CreateSession(poll, id, body(flask.request, 'user_id')).run()




api.add_resource(PollEndpoint, '/<string:poll>')
api.add_resource(InstrumentsEndpoint, '/<string:poll>/instrument')
api.add_resource(UsersEndpoint, '/<string:poll>/user')
api.add_resource(UserEndpoint, '/<string:poll>/user/<int:id>')
api.add_resource(DatesEndpoint, '/<string:poll>/date')
api.add_resource(DateEndpoint, '/<string:poll>/date/<int:id>')
api.add_resource(AnswersEndpoint, '/<string:poll>/answer')
api.add_resource(AnswerEndpoint, '/<string:poll>/answer/<int:id>')
api.add_resource(GetAnswerEndpoint, '/<string:poll>/answer/<int:userId>/<int:dateId>')
api.add_resource(SessionEndpoint, '/<string:poll>/session/<string:id>')

def run():
    app.run(debug=True, host='0.0.0.0', port=4444)

if __name__ == '__main__':
    run()



import sqlite3

from popoll_backend.model import Payload
from popoll_backend.model.payload.id_payload import IdPayload
from popoll_backend.query import Query


class CreateInstrument(Query):
    
    name: str
    instrument_id: int
    
    def __init__(self, poll, name: str):
        super().__init__(poll)
        self.name = name
        
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('INSERT INTO instruments(name) VALUES (?)', (self.name,))
        self.instrument_id = cursor.lastrowid
        
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return IdPayload(self.instrument_id)



import sqlite3

import flask

from popoll_backend.model import Payload
from popoll_backend.query import Query
from popoll_backend.query.get_answer import GetAnswer


class CreateAnswer(Query):
    
    user_id: int
    date_id: int
    
    id: int
    
    def __init__(self, poll:str, user_id: int, date_id: int):
        super().__init__(poll)
        self.user_id = user_id
        self.date_id = date_id
    
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('INSERT INTO answers(user_id, date_id, response) VALUES (?, ?, ?)', (self.user_id, self.date_id, True))
        self.id = cursor.lastrowid
        
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return GetAnswer(self.poll, self.id).run(cursor)
    
    def error(self, e: sqlite3.Error):
        if isinstance(e, sqlite3.IntegrityError):
            if e.sqlite_errorcode == sqlite3.SQLITE_CONSTRAINT_UNIQUE:
                flask.abort(400, f'Answer for this user already exists. PUT should be triggered instead.')


import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.model.db.answer import Answer
from popoll_backend.model.db.date import Date
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user import User
from popoll_backend.model.db.user_instruments import UserInstruments
from popoll_backend.model.payload.date_payload import DatePayload
from popoll_backend.query import Query


class GetDate(Query):
    
    id: int
    details: bool
    
    date: Date
    answers: List[Answer]
    users: List[User]
    instruments: List[Instrument]
    user_instruments: List[UserInstruments]
    
    def __init__(self, poll: str, id: int, details: bool = False):
        super().__init__(poll)
        self.id = id
        self.details = details

    def process(self, cursor: sqlite3.Cursor):
        self.date = self.selectItem(cursor, 'SELECT * from dates where id=?', self.id, Date)
        if self.details:
            self.answers: List[Answer] = [Answer(row) for row in cursor.execute('SELECT * from answers where date_id=?', (self.id,)).fetchall()]
            self.users: List[User] = [User(row) for row in cursor.execute('SELECT * from users ORDER BY name').fetchall()]
            self.instruments: List[Instrument] = [Instrument(row) for row in cursor.execute('SELECT * FROM instruments').fetchall()]
            self.user_instruments: List[UserInstruments] = [UserInstruments(row) for row in cursor.execute('SELECT * FROM user_instruments').fetchall()]

    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        if not self.details:
            return self.date
        return DatePayload(self.date, self.answers, self.users, self.instruments, self.user_instruments)


from tests import IntegrationTest, TestForwardError

class TestUpdateDateEntry(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.date2_id = self.create_date(date='2025-03-12', time=None, title='secondDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        
    def test_update_date_entry(self):
        updated_date1_id = self.udpate_date(self.date1_id, date='2025-03-10', time='18:00:00', title='firstDateUpdated')
        assert updated_date1_id == self.date1_id
        _json = self.get_user(self.user1_id)
        assert _json['dates'][0]['date']['date'] == '2025-03-10'
        assert _json['dates'][0]['date']['time'] == '18:00:00'
        assert _json['dates'][0]['date']['title'] == 'firstDateUpdated'
        
    def test_update_date_entry_remove_time(self):
        updated_date1_id = self.udpate_date(self.date1_id, date='2025-03-10', time=None, title='firstDateUpdated')
        assert updated_date1_id == self.date1_id
        _json = self.get_user(self.user1_id)
        assert _json['dates'][0]['date']['date'] == '2025-03-10'
        assert _json['dates'][0]['date']['title'] == 'firstDateUpdated'
        
     # TODO Does not make sens since is returning valid answer but nothing impacted  
    # def test_update_date_entry_not_found(self):
    #     try:
    #         self.udpate_date(self.date1_id + 10, date='2025-03-10', time='18:00:00', title='firstDateUpdated')
    #         assert False
    #     except TestForwardError as e:
    #         assert e.rs.status_code == 404
    #         _json = self.toJSON(self.get())
    #         assert len(_json['users']) == 1



import sqlite3
from typing import List

import flask

from popoll_backend.model import Payload
from popoll_backend.query import Query
from popoll_backend.query.get_user import GetUser


class UpdateUser(Query):
    
    id: int
    name: str
    main_instrument: int
    instruments: List[int]
    
    def __init__(self, poll, id, name, main_instrument, instruments):
        super().__init__(poll)
        self.id = id
        self.name = name
        self.main_instrument = main_instrument
        self.instruments = instruments
    
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('UPDATE users SET name=? WHERE id=?', (self.name, self.id))
        cursor.execute('DELETE FROM user_instruments WHERE user_id=?', (self.id,))
        rows = [(self.id, self.main_instrument, True)] + [(self.id, instru, False) for instru in self.instruments if not instru == self.main_instrument]
        cursor.executemany('INSERT INTO user_instruments(user_id, instrument_id, is_main) VALUES(?, ?, ?)', rows)
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return GetUser(self.poll, self.id, details=False).run(cursor)
    
    def error(self, e: sqlite3.Error):
        if isinstance(e, sqlite3.IntegrityError):
            if e.sqlite_errorcode == sqlite3.SQLITE_CONSTRAINT_UNIQUE:
                flask.abort(400, f'User with name {self.name} already exists, cannot update.')


from tests import IntegrationTest

class TestAddAnswerNoDupe(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.date2_id = self.create_date(date='2025-03-11', time=None, title='secondDate')

    def _assert_date(self, e_id, e_date, e_time, e_title, actual):
        assert e_id == actual['id']
        assert e_title == actual['title']
        assert e_date == actual['date']
        assert e_time == actual['time']

    def test_get_dates(self):
        rs = self.get_dates()
        assert len(rs['dates']) == 2
        self._assert_date(self.date1_id, '2025-03-10', '15:00:00', 'firstDate', rs['dates'][0])
        self._assert_date(self.date2_id, '2025-03-11', None, 'secondDate', rs['dates'][1])
        




from tests import IntegrationTest

class TestAddAnswerNoDupe(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.answer1_id = self.create_answer(self.user1_id, self.date1_id)


    def test_delete_database(self):            
        rs = self.create_answer(self.user1_id, self.date1_id, fail=True)
        assert rs.status_code == 400
        




import sqlite3

import flask

from popoll_backend.model import Payload
from popoll_backend.model.db.session import Session
from popoll_backend.model.payload.id_payload import IdPayload
from popoll_backend.query import Query


class GetSession(Query):
    
    id: int
    
    session: Session
    
    def __init__(self, poll: str, id: str):
        super().__init__(poll)
        self.id = id

    def process(self, cursor: sqlite3.Cursor):
        pass

    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        res = cursor.execute('SELECT * FROM sessions WHERE session_id=?', (self.id,)).fetchall()
        if len(res) == 0:
            flask.abort(404, f'Object of type [Session] and session_id [{self.id}] has not been found.')
        return Session(res[-1])


from tests import IntegrationTest

def _test_instru(expected_id, expected_name, isMain, actual):
        assert expected_id == actual['id']
        assert expected_name == actual['name']
        # assert isMain == actual['is_main']

class TestMultiInstruments(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [self.instru2_id])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        _json = self.get_user(self.user1_id)
        _test_instru(self.instru1_id, self.INSTRU1, True, _json['user']['main_instrument'])
        instruments = _json['user']['instruments']
        assert len(instruments) == 1
        _test_instru(self.instru2_id, self.INSTRU2, False, instruments[0])
        
    def test_multi_instruments_update_add(self):
        user1_id = self.update_user(self.user1_id, 'updatedUser1', self.instru1_id, [self.instru2_id, self.instru3_id])
        assert user1_id == self.user1_id
        _json = self.get_user(self.user1_id)
        _test_instru(self.instru1_id, self.INSTRU1, True, _json['user']['main_instrument'])
        instruments = _json['user']['instruments']
        assert len(instruments) == 2
        _test_instru(self.instru2_id, self.INSTRU2, False, instruments[0])
        _test_instru(self.instru3_id, self.INSTRU3, False, instruments[1])

    def test_multi_instruments_update_move(self):
        user1_id = self.update_user(self.user1_id, 'updatedUser1', self.instru2_id, [self.instru1_id])
        assert user1_id == self.user1_id
        _json = self.get_user(self.user1_id)
        _test_instru(self.instru2_id, self.INSTRU2, True, _json['user']['main_instrument'])
        assert len(_json['user']['instruments']) == 1
        _test_instru(self.instru1_id, self.INSTRU1, False, _json['user']['instruments'][0]) # Since is a set, order is a bit unpredictable

    def test_multi_instruments_update_delete(self):
        user1_id = self.update_user(self.user1_id, 'updatedUser1', self.instru1_id, [])
        assert user1_id == self.user1_id
        _json = self.get_user(self.user1_id)
        _test_instru(self.instru1_id, self.INSTRU1, True, _json['user']['main_instrument'])
        assert len(_json['user']['instruments']) == 0


from typing import Any, List

from popoll_backend.model import Payload

class Answer(Payload):
    
    def __init__(self, data: List[Any]):
        self.id: int = data[0]
        self.user_id: int = data[1]
        self.date_id: int = data[2]
        self.response: bool = bool(data[3])

    @classmethod
    def create_table(cls):
        return """ CREATE TABLE IF NOT EXISTS answers (
            id integer PRIMARY KEY AUTOINCREMENT,
            user_id integer NOT NULL REFERENCES users(id) ON DELETE CASCADE,
            date_id integer NOT NULL REFERENCES dates(id) ON DELETE CASCADE,
            response boolean NOT NULL,
            UNIQUE(user_id, date_id)
        ); """


import sqlite3
from typing import List

from popoll_backend.model import Payload
from popoll_backend.model.db.option import Option
from popoll_backend.query import Query


class GetPoll(Query):
    
    def __init__(self, poll: str):
        super().__init__(poll)

    def process(self, cursor: sqlite3.Cursor):
        pass
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        res = cursor.execute('SELECT * FROM options').fetchone()
        return Option(res)


from tests import IntegrationTest

def test_line(json, method=None, ttype=None):
    assert json['method'] == method
    assert json['_type'] == ttype

class TestHistory(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.user2_id = self.create_user('user2', self.instru2_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.date2_id = self.create_date(date='2025-03-11', time='18:00:00', title='secondDate')
        self.date3_id = self.create_date(date='2025-03-12', time=None, title='thirdDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id, response=True)
        self.answer2_id = self.create_answer(user_id=self.user1_id, date_id=self.date2_id, response=True)
        self.answer3_id = self.create_answer(user_id=self.user1_id, date_id=self.date3_id, response=True)
        self.answer4_id = self.create_answer(user_id=self.user2_id, date_id=self.date1_id, response=True)
        self.answer5_id = self.create_answer(user_id=self.user2_id, date_id=self.date2_id, response=True)
        self.answer6_id = self.create_answer(user_id=self.user2_id, date_id=self.date3_id, response=True)
        self.udpateAnswer(self.answer1_id, False)
        self.delete_date(self.date2_id)

    def expected_lines(self, nbCreateInstrus, nbCreateUsers, nbCreateDates, nbCreateAnswers, nbUpdate, nbDelete):
        return nbCreateInstrus + nbCreateUsers*2 + nbCreateDates + nbCreateAnswers + nbUpdate + nbDelete
        
    # def test_get_user_sorted(self):
    #     _json = self.toJSON(self.get_history())['history']
    #     assert self.expected_lines(3, 2, 3, 6, 1, 1) == len(_json)
    #     test_line(_json[0], method='add', ttype='Instrument')
    #     test_line(_json[1], method='add', ttype='Instrument')
    #     test_line(_json[2], method='add', ttype='Instrument')
    #     test_line(_json[3], method='add', ttype='User')
    #     test_line(_json[4], method='update', ttype='User')
    #     test_line(_json[5], method='add', ttype='User')
    #     test_line(_json[6], method='update', ttype='User')
    #     test_line(_json[7], method='add', ttype='Date')
    #     test_line(_json[8], method='add', ttype='Date')
    #     test_line(_json[9], method='add', ttype='Date')
    #     test_line(_json[10], method='add', ttype='Answer')
    #     test_line(_json[11], method='add', ttype='Answer')
    #     test_line(_json[12], method='add', ttype='Answer')
    #     test_line(_json[13], method='add', ttype='Answer')
    #     test_line(_json[14], method='add', ttype='Answer')
    #     test_line(_json[15], method='add', ttype='Answer')
    #     test_line(_json[16], method='update', ttype='Answer')
    #     test_line(_json[17], method='delete', ttype='Date')
        



from typing import Any, List
from popoll_backend.model import Payload
from popoll_backend.model.db.user import User
from popoll_backend.model.db.instrument import Instrument
from popoll_backend.model.db.user_instruments import UserInstruments


class UserOut(Payload):
    
    def __init__(self, user: User, instruments: Instrument, user_instruments: List[UserInstruments]):
        self.user = user
        self.main_instrument: Instrument = [self.toDict(instruments)[i.instrument_id] for i in user_instruments if i.user_id == user.id and i.is_main][0]
        self.instruments: List[Instrument] = [self.toDict(instruments)[i.instrument_id] for i in user_instruments if i.user_id == user.id and not i.is_main]


import sqlite3
from typing import Any, Dict, Tuple, Type

import flask

from popoll_backend.model import Payload
from popoll_backend.model.payload.id_payload import IdPayload

class Query:
    
    def __init__(self, poll: str):
        self.poll = poll
    
    def run(self, cursor: sqlite3.Cursor=None):
        if cursor:
            return self._run(cursor)
        else:
            with sqlite3.connect(f'{self.poll}.db') as connection:
                cursor: sqlite3.Cursor = connection.cursor()
                cursor.execute("PRAGMA foreign_keys = ON")
                return self._run(cursor).toJSON()
            
    def _run(self, cursor: sqlite3.Cursor):
        try:
            self.process(cursor)
            return self.buildResponse(cursor)
        except sqlite3.Error as e:
            print(e)
            self.error(e)
            if e.sqlite_errorcode == sqlite3.SQLITE_ERROR:
                flask.abort(500, 'Main error. Are you sure database exists?')
    
    def process(self, cursor: sqlite3.Cursor) -> None:
        raise NotImplementedError()
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        raise NotImplementedError()
    
    def error(self, e: sqlite3.Error):
        pass
    
    def selectItem(self, cursor: sqlite3.Cursor, query: str, id: int, type: Type[Payload]) -> Payload:
        res = cursor.execute(query, (id,)).fetchall()
        if len(res) == 0:
            flask.abort(404, f'Object of type [{type.__name__}] and id [{id}] has not been found.')
        if len(res) > 1:
            print('\n\t'.join([
                'More than 1 result for following query. This is not expected.',
                f'query={query}',
                f'id={id}',
                f'res={res}'
            ]))
        return type(res[-1])


import sqlite3
from typing import Optional

from popoll_backend.model import Payload
from popoll_backend.model.db.answer import Answer
from popoll_backend.query import Query
from popoll_backend.query.get_answer import GetAnswer


class UpdateAnswer(Query):
    
    id: int
    response: Optional[bool]
    
    def __init__(self, poll:str, id: int, response: Optional[bool]):
        super().__init__(poll)
        self.id = id
        self.response = response
    
    def process(self, cursor: sqlite3.Cursor):
        cursor.execute('UPDATE answers SET response=? WHERE id=?', (self.response, self.id))
    
    def buildResponse(self, cursor: sqlite3.Cursor) -> Payload:
        return GetAnswer(self.poll, self.id).run(cursor)


import json

from tests import IntegrationTest, TestForwardError

class TestNoDuplicates(IntegrationTest):

    def setUp(self):
        super().setUp()
        self.user1_id = self.create_user('user1', self.instru1_id, [])
        self.date1_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        self.answer1_id = self.create_answer(user_id=self.user1_id, date_id=self.date1_id)
        
    def test_no_duplicate_user(self):
        rs = self.create_user('user1', self.instru1_id, [], fail=True)
        assert rs.status_code == 400
        # TODO assert json.loads(rs.json)["message"] == 'User with name user1 already exists.'
    
    def test_no_duplicate_user_then_valid(self):
        rs = self.create_user('user1', self.instru1_id, [], fail=True)
        assert rs.status_code == 400
        user2_id = self.create_user('user2', self.instru2_id, [])
        _json = self.get_users()
        assert len(_json['users']) == 2

            
    def test_can_duplicate_dates(self):
        date2_id = self.create_date(date='2025-03-10', time='15:00:00', title='firstDate')
        _json = self.get_user(self.user1_id)
        assert len(_json['dates']) == 2
        assert self.date1_id != date2_id